
# q-expansion of newform 6864.2.a.y, downloaded from the LMFDB on 07 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6864
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [2], [-4], [1], [-1], [2], [4], [-4], [-2], [0], [-2], [10], [4], [4], [2], [4], [10], [4], [-12], [2], [4], [12], [2], [2], [6], [16], [4], [-14], [-14], [20], [-12], [10], [12], [-10], [20], [-10], [-20], [-16], [-2], [12], [-2], [12], [18], [14], [16], [12], [16], [-12], [6], [26], [24], [2], [-12], [18], [-24], [-6], [-28], [2], [-6], [-12], [-10], [-12], [-20], [-6], [2], [20], [-14], [12], [34], [-6], [32], [0], [26], [20], [28], [18], [-18], [18], [-6], [-20], [-34], [16], [-30], [-36], [-28], [18], [-22], [-2], [-24], [20], [0], [-16], [36], [-4], [0], [-6], [2], [-28], [18], [-44], [30], [-36], [10], [-4], [34], [36], [18], [-36], [26], [-44], [34], [-30], [28], [24], [42], [-44], [12], [-46], [-4], [22], [10], [6], [4], [-12], [-10], [-26], [20], [-16], [2], [44], [48], [0], [-42], [-22], [18], [-30], [-36], [26], [42], [-12], [30], [-40], [4], [14], [-20], [34], [-38], [-4], [36], [50], [-30], [44], [-48], [-28], [-28], [-28], [18], [-6], [-42], [36], [-6], [20], [4], [18], [-44], [16], [-6], [-46], [-22], [36], [-34], [-48], [18], [-16], [26], [-20], [-54], [20], [50], [-16], [-12], [-10], [10], [-52], [58], [-54], [20], [-6], [-48], [-46], [-60], [20], [42], [-36], [-6], [-38], [-26], [66], [-24], [62], [12], [22], [50], [-12], [18], [40], [4], [-6], [-60], [42], [-6], [-56], [12], [-16], [-22], [4], [-14], [68], [2], [2], [-28], [-30], [-64], [-36], [42], [-54], [12], [-68], [12], [14], [60], [52], [10], [-20], [0], [66], [-74], [20], [4], [36], [52], [-56], [30], [-46], [-16], [-8], [36], [-4], [40], [-54], [-14], [-52], [58], [-74], [-20], [38], [-28], [42], [-38], [-68], [-28], [-70], [34], [66], [12], [-30], [-30], [44], [38], [-10], [4], [10], [4], [34], [-16], [-36], [-54], [-30], [28], [16], [64], [32], [10], [76], [-60], [-30], [-82], [32], [-30], [10], [-4], [-38], [-20], [-14], [-10], [24], [-46], [60], [-68], [-78], [-18], [-68], [-76], [20], [-78], [-4], [-34], [28], [26], [-8], [58], [18], [36], [88], [10], [4], [0], [34], [-30], [52], [26], [-42], [-88], [58], [-14], [68], [20], [72], [-26], [26], [42], [-4], [36], [-92], [28], [14], [2], [58], [68], [70], [2], [-26], [-8], [26], [-44], [-66], [52], [24], [34], [-36], [-70], [-54], [-76], [18], [74], [-20], [12], [18], [12], [34], [-54], [-12], [28], [44], [10], [-34], [-12], [-62], [60], [76], [-64], [6], [28], [-2], [-28], [8], [-86], [-46], [18], [42], [42], [36], [18], [12], [44], [0], [86], [-44], [44], [-14], [-6], [-60], [52], [60], [-14], [-28], [-30], [44], [-50], [-70], [-54], [-44], [26], [86], [-28], [-42], [50], [-76], [36], [-94], [46], [-84], [-60], [90], [66], [104], [96], [50], [64], [2], [-30], [-36], [-12], [-54], [42], [84], [-22], [-28], [24]]
